/*******************************************************************************
 * time.C - Time class
 *
 * $Id: time.C,v 2.0 2001/05/23 11:49:10 mscheid Exp $
 *
 * Author: Matthias Scheidegger <mscheid@iam.unibe.ch>
 *
 * Copyright 2001, Matthias Scheidegger
 *
 ******************************************************************************/

#include "time.H"
#include <math.h>

const __u32 unix2ntp = 2208988800UL;   // seconds between 1900 and 1970
const double pow_2_32 = 4294967296.0;  // 2^32


void Time::set()
{
    struct timeval tv;
    if (gettimeofday(&tv, NULL)) do_throw<Exception>(errno, "gettimeofday");
    set_to(&tv);
}


void Time::set_to(double seconds)
{
    t = seconds;
}


void Time::set_to(__u64 ntpt)
{
    t = (ntpt >> 32) - unix2ntp;          // integral part
    t += (ntpt & 0xffffffff) / pow_2_32;  // fractional part
}


void Time::set_to(const struct timeval *tv)
{
    t = tv->tv_sec;
    t += tv->tv_usec / 1000000.0;
}


void Time::set_to(const struct timespec *ts)
{
    t = ts->tv_sec;
    t += ts->tv_nsec / 1000000000.0;
}


__u64 Time::get_ntp() const
{
    double secs;
    double frac;
    __u64 res;
    
    if (t < 0) do_throw<Assertion>("Time must be positive for conversion to NTP");

    frac = modf(t, &secs);
    secs += unix2ntp;
    frac *= pow_2_32;

    res = (__u64)secs;
    res <<= 32;
    res |= (__u32)rint(frac);
    return res;

}


__u32 Time::get_trunc_ntp() const
{
    __u64 nt;

    nt = get_ntp();
    nt >>= 16;
    nt &= 0xffffffff;
    return nt;
}


struct timeval* Time::get_timeval(struct timeval *tv) const
{
    double secs;
    double frac;

    if (t < 0) do_throw<Assertion>("Time must be positive for conversion to timeval");

    frac = modf(t, &secs);
    tv->tv_sec = (__time_t)secs;
    tv->tv_usec = (__time_t)(frac * 1000000);
    return tv;
}


struct timespec* Time::get_timespec(struct timespec *ts) const
{
    double secs;
    double frac;

    if (t < 0) do_throw<Assertion>("Time must be positive for conversion to timespec");

    frac = modf(t, &secs);
    ts->tv_sec = (__time_t)secs;
    ts->tv_nsec = (__time_t)(frac * 1000000000);
    return ts;
}
